
from requests import Response


def check_status_code(req: Response):
    """
    Check status code of requests response to raise the correct exception

    :param req: The requests Response of your requests call
    """

    if req.status_code == 400:
        raise QueryError(status_code=400, url=req.url, message="Endpoint is invalid or was built incorrectly")
    elif req.status_code == 401:
        raise JWTError(status_code=401, message="JWT Access token is missing or invalid credentials were provided")
    elif req.status_code == 403:
        raise PermissionsError(status_code=403, message="User doesn't have the correct permissions to access this data")
    elif req.status_code == 404:
        raise QueryError(status_code=404, url=req.url, message="Endpoint is invalid or was built incorrectly")


class PyBehrtechException(Exception):
    """Base exception used by this module."""
    pass


class JWTError(PyBehrtechException):
    """A request for the JWT failed"""

    def __init__(self, status_code, message):
        self.status_code = status_code
        PyBehrtechException.__init__(self, status_code, message)


class PermissionsError(PyBehrtechException):
    """User of request doesn't have permissions"""

    def __init__(self, status_code, message):
        self.status_code = status_code
        PyBehrtechException.__init__(self, status_code, message)


class QueryError(PyBehrtechException):
    """Request to endpoint is invalid or was built incorrectly"""

    def __init__(self, status_code, url, message):
        self.status_code = status_code
        self.url = url
        PyBehrtechException.__init__(self, status_code, url, message)



import requests

from requests.exceptions import RequestException
from urllib3.exceptions import MaxRetryError, ConnectTimeoutError
from py_behrtech.parsers import Parser
from py_behrtech.exceptions import JWTError, check_status_code


class System:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def authTicketGet(self) -> str:
        """
        Get ticket information from which you can request an upgrade to a websocket connection at /ws

        :return: Parser object of ticket information
        """

        req = requests.get(url=self.server_address + f"/v2/auth/ticket", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json().get("ticket")
        else:
            check_status_code(req=req)

    def systemEulaGet(self) -> str:
        """
        Gets the end user license agreement

        :return: The end user license agreement
        """

        req = requests.get(url=self.server_address + f"/v2/system/eula", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json().get("EULA")
        else:
            check_status_code(req=req)

    def systemGet(self) -> Parser:
        """
        Gets system status information

        :return: Parser object of system status information
        """

        req = requests.get(url=self.server_address + f"/v2/system", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return Parser(req=req)
        else:
            check_status_code(req=req)

    def wsGet(self, access_token: str = None) -> Parser:
        """
        Request an upgrade to a websocket connection (Handshake is handled by goland http upgrader)

        :param access_token: Unique token to authenticate to for requesting an upgrade to a websocket connection
        :return: Parser object of a websocket connection upgrade
        """
        if not access_token:
            access_token = self.authTicketGet()

        req = requests.get(url=self.server_address + f"/v2/ws?accessToken={access_token}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return Parser(req=req)
        else:
            check_status_code(req=req)



import requests

from py_behrtech.parsers import Parser
from py_behrtech.exceptions import check_status_code


class Nodes:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def nodesEpEuiGet(self, ep_eui: str):
        """
        Gets information on a configured node with the matching EpEui

        :param ep_eui: The unique epEui number of the node to be requested
        :return: Parser object of the configured node
        """
        req = requests.get(url=self.server_address + f"/v2/nodes/{ep_eui}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return Parser(req=req)
        else:
            check_status_code(req=req)

    def nodesGet(self, return_count: int = '', offset: int = '', sensor_type: str = ''):
        """
        Gets information on all configured nodes

        :param return_count: The amount of nodes to be requested. (-1 for all)
        :param offset: Node amount to start the request from
        :param sensor_type: Node type to be requested
        :return: Parser object of configured nodes
        """
        parameters = ''

        if return_count:
            parameters += f"?returnCount={return_count}"
        if offset:
            parameters += (f"&offset={offset}" if parameters else f"?offset={offset}")
        if sensor_type:
            parameters += (f"&sensorType={sensor_type}" if parameters else f"?sensorType={sensor_type}")

        req = requests.get(url=self.server_address + f"/v2/nodes" + parameters, verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return Parser(req=req)
        else:
            check_status_code(req=req)



import requests

from requests.exceptions import RequestException
from urllib3.exceptions import MaxRetryError, ConnectTimeoutError
from py_behrtech.exceptions import JWTError, check_status_code


class User:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def login(self) -> str:
        """
        Gets the JWT token of the login endpoint

        :return: The JWT token of the login endpoint
        """

        try:
            req = requests.post(url=self.server_address + "/v2/login", verify=False,
                                json={'user': self.username, 'password': self.password}, timeout=3)

            if req.status_code == 200:
                # Successfully logged in
                return req.json().get("JWT")
            else:
                check_status_code(req=req)

        except (TimeoutError, ConnectTimeoutError, MaxRetryError, RequestException):
            raise JWTError(status_code=401, message="The wrong server IP address or login credentials were provided")

    def userGet(self) -> dict:
        """
        Retrieves all registered users, only accessible for admin users

        :return: All registered users
        """

        req = requests.get(url=self.server_address + f"/v2/user", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.text
        else:
            check_status_code(req=req)

    def userPost(self, name: str, email: str, admin: bool, getMethodsOnly: bool) -> dict:
        """
        Adds a new user, only accessible for admin users

        :param name: Name of the user account
        :param email: Email for the user account
        :param admin: If user account is a admin
        :param getMethodsOnly:
        :return: Newly create user account information
        """

        req = requests.post(url=self.server_address + f"/v2/user", verify=False,
                            headers={"Authorization": f"Bearer {self.jwt_token}"},
                            json={'name': name, 'email': email, 'admin': admin, 'getMethodsOnly': getMethodsOnly},
                            timeout=3)

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def userUUIDDelete(self, uuid: str) -> bool:
        """
        Deletes the specified user, only accessible for admin users

        :param uuid: Unique user ID
        :return: Bool if user has been successfully deleted or not
        """

        req = requests.delete(url=self.server_address + f"/v2/user/{uuid}", verify=False,
                              headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return True
        else:
            check_status_code(req=req)

    def userUUIDGet(self, uuid: str) -> dict:
        """
        Retrieves user information for the specified user, only accessible for admin users

        :param uuid: Unique user ID
        :return: User profile information
        """

        req = requests.get(url=self.server_address + f"/v2/user/{uuid}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return req.json()
        else:
            check_status_code(req=req)

    def userUUIDProfilePost(self, uuid: str, **kwargs) -> bool:
        """
        Updates user profile information

        :param uuid: Unique user ID
        :keyword name: Name of user account
        :keyword email: Email of user account
        :keyword oldPassword: Old password of user account
        :keyword newPassword: New password of user account
        :keyword acceptedEULA: If EULA has been accepted
        :return: Bool if user profile information has been updated or not
        """

        allowed = ['name', 'email', 'oldPassword', 'newPassword', 'acceptedEULA']
        body = {x: kwargs[x] for x in kwargs if x in allowed and kwargs[x] is not None}

        req = requests.post(url=self.server_address + f"/v2/user/{uuid}/profile", verify=False,
                            headers={"Authorization": f"Bearer {self.jwt_token}"}, json=body, timeout=3)

        if req.status_code == 200:
            return True
        else:
            check_status_code(req=req)

    def userUUIDResetPasswordPost(self, uuid: str) -> str:
        """
        Resets a users password to a randomly generated one, only accessible for admin users

        :param uuid: Unique user ID
        :return: New reset password for user
        """

        req = requests.post(url=self.server_address + f"/v2/user/{uuid}/resetpassword", verify=False,
                            headers={"Authorization": f"Bearer {self.jwt_token}"}, timeout=3)

        if req.status_code == 200:
            return req.json().get("password")
        else:
            check_status_code(req=req)



import requests

from py_behrtech.parsers import Parser
from py_behrtech.exceptions import check_status_code


class Models:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def models_delete(self):
        pass

    def models_get(self, return_count: int = '', offset: int = ''):
        """
        Gets information on all registered node models

        :param return_count: The amount of node models to be requested. (-1 for all)
        :param offset: Node model amount to start the request from
        :return: Parser object of configured node models
        """

        parameters = ''

        if return_count:
            parameters += f"?returnCount={return_count}"
        if offset:
            parameters += (f"&offset={offset}" if parameters else f"?offset={offset}")

        req = requests.get(url=self.server_address + f"/v2/sensormodels" + parameters, verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return Parser(req=req)
        else:
            check_status_code(req=req)

    def models_post(self):
        pass

    def models_type_delete(self):
        pass

    def models_type_get(self, model_type: str):
        """
        Gets information on a registered node model

        :param model_type: Unique identifier of the node model to be requested
        :return: Parser object of configured node model
        """

        req = requests.get(url=self.server_address + f"/v2/sensormodels/{model_type}", verify=False,
                           headers={"Authorization": f"Bearer {self.jwt_token}"})

        if req.status_code == 200:
            return Parser(req=req)
        else:
            check_status_code(req=req)

    def models_type_post(self):
        pass
